<?php
/**
 * Parasite
 * API для получения списков Parasite Eliminator (http://parasite-eliminator.ru/)
 *
 * Для добавления URL в список: addtoblacklist@parasite-eliminator.ru
 *
 * @author Александр Макаров (http://rmcreative.ru/)
 */

class ParasiteException extends Exception {};

class ParasiteApi {
	const VERSION = 'beta1';

	const FILE_SIGNATURE = 'Parasite Eliminator Update';

	const SERVER_PRIMARY = 'http://parasite-eliminator.ru/updates/update.txt';
	const SERVER_SECONDARY = 'http://parasite-eliminator.narod.ru/updates/update.txt';

	/**
	 * Получает список и отдаёт массивом
	 * @return array
	 */
	public function getList() {
		$lines = $this->getLines(self::SERVER_PRIMARY);
		if(!$this->isValidUpdate($lines)) {
			$lines = $this->getLines(self::SERVER_SECONDARY);
			if(!$this->isValidUpdate($lines)) {
				throw new ParasiteException("Не удалось получить данные с сервера.");
			}
		}

		return $this->parse($lines);
	}

	/**
	 * Разбирает список
	 * @param string $lines
	 * @return array
	 */
	private function parse($lines) {
		$result = array();

		foreach ($lines as $line) {
			$line = ltrim($line);

			// Если в начале строки VERSION — это версия базы
			if (strpos($line, 'VERSION') === 0) {
				$result['version'] = trim(str_replace('VERSION', '', $line));
			}

			// Если в начале строки минус — в черный список
			if (strpos($line, '-') === 0) {
				$line = self::normalizeUrl(ltrim($line, '-'));

				if (self::isDomainName($line)) {
					$result['black'][] = $line;
				}
			}

			// Если в начале строки плюс — в белый список
			if ( strpos($line, '+') === 0 ) {
				$line = self::normalizeUrl(ltrim($line, '+'));

				if (self::isDomainName($line)) {
					$result['white'][] = $line;
				}
			}
		}

		return $result;
	}

	/**
	 * Приводит URL к виду blog.site.ru
	 *
	 * @param string $url
	 * @return string
	 */
	static function normalizeUrl($url) {
		$url = trim($url);

		if(empty($url)) return '';

		$url = strtolower($url);

		preg_match('~(?:http://)?(?:www.)?([^?/&]+)~', $url, $matches);
		$url = $matches[1];

		return $url;
	}

	/**
	 * Является ли строка доменным именем
	 * @param string $url
	 * @return boolean
	 */
	static function isDomainName($url) {
		if(empty($url)) return false;
		return preg_match(
		'/^(?:(?:w{3}\.)?(?:[a-z0-9_-]*?\.){1,2}[a-z]{2,4}(?:\/[a-z0-9_-]*?)*?(?:\.[a-z~]{1,5})?\??(?:[a-z_-]+?=[a-z0-9_\%-]+?)*?(?:\&[a-z]=[a-z0-9_\%-]*)*?)?$/i',
		$url
		);
	}

	/**
	 * Загрузка списка с сервера
	 *
	 * @param string $url
	 * @return string
	 */
	private function getLines($url) {
		if(function_exists('curl_init')) {
			return $this->getUpdateViaCurl($url);
		}
		else {
			@ini_set('allow_url_fopen', 1);
			if(ini_get('allow_url_fopen')) {
				@ini_set('user_agent', $this->getUserAgent());
				return @file($url);
			}
		}
		throw new ParasiteException("fopen и curl отключены. Получение списка невозможно.");
	}

	/**
	 * Проверяет список на повреждения
	 *
	 * @param string $lines
	 * @return boolean
	 */
	private function isValidUpdate($lines) {
		if(empty($lines)) return false;

		$open = trim($lines[0]);
		$close = trim($lines[count($lines) - 1]);

		return self::FILE_SIGNATURE==$open && self::FILE_SIGNATURE==$close;
	}

	/**
	 * Отдаёт user agent
	 * @return string
	 */
	private function getUserAgent() {
		return 'Parasite Eliminator API /Version ' . self::VERSION . ' /Base ' . '-'  . ' /Updated ' . '-' . ' /Trashed ' . '-' . getenv('SERVER_NAME');
	}

	/**
	 * Получаем список curl-ом
	 *
	 * @param string $url
	 * @return string
	 */
	private function getUpdateViaCurl($url) {
		$ch = curl_init();
		$timeout = 3;

		curl_setopt ($ch, CURLOPT_USERAGENT, $this->getUserAgent());
		curl_setopt ($ch, CURLOPT_URL, $url);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		$out = curl_exec($ch);

		curl_close($ch);

		return explode("\n", $out);
	}
}